@@ -31,9 +31,9 @@ use Sys::Info::Constants qw( :device_cpu );
 
 use Exporter 'import';
 our @EXPORT = qw(system_identity suite_run calc_scalability);
-my $datadir = dist_dir("Benchmark-DKbench");
+our $datadir = dist_dir("Benchmark-DKbench");
 
-our $VERSION = '2.0_002';
+our $VERSION = '2.0_003';
 
 =head1 NAME
 
@@ -62,8 +62,8 @@ and XS) workloads. Runs single and multi-threaded and can be fully customized to
 the benchmarks that better suit your own scenario.
 
 The only non-CPAN libraries required to install/run the suite are libdb, libjpeg
-(with headers). There are a few prerequisite CPAN libraries to load suite, plus
-the optional BioPerl (requires libxml on system) which enables extra benchmarks.
+(with headers). There are a few prerequisite CPAN libraries, plus an optional one
+(L<BioPerl>, requires libxml on system) which enables extra benchmarks.
 
 =head1 SCRIPTS
 
@@ -277,48 +277,26 @@ the same terms as the Perl 5 programming language system itself.
 =cut
 
 sub benchmark_list {
-    my $quick = shift;
-    return {
-        'Astro'            => sub {bench_astro($quick ? 20000 : 80000) },
-        'BioPerl Codons'   => sub {bench_bioperl_codons(1, $quick ? 3 : 5) },
-        'BioPerl Monomers' => sub {bench_bioperl_mono($quick ? 6 : 20) },
-        'Crypt::JWT'       => sub {bench_jwt($quick ? 250 : 900) },
-        'CSS::Inliner'     => sub {bench_css($quick ? 2 : 5) },
-        'DateTime'         => sub {bench_datetime($quick ? 5000 : 15000) },
-        'Digest'           => sub {bench_digest($quick ? 50 : 250) },
-        'Encode'           => sub {bench_encode($quick ? 40 : 120) },
-        'HTML::FormatText' => sub {bench_formattext($quick ? 4 : 10) },
-        'Imager'           => sub {bench_imager($quick ? 4 : 16) },
-        'JSON::XS'         => sub {bench_json($quick ? 600 : 2200) },
-        'Math::DCT'        => sub {bench_dct($quick ? 25000 : 100_000) },
-        'Math::MatrixReal' => sub {bench_matrixreal($quick ? 200 : 650) },
-        'Moose'            => sub {bench_moose($quick ? 10_000 : 30_000) },
-        'Moose prove'      => sub {bench_moose_tests($quick ? 0.5 : 1) },
-        'Primes'           => sub {bench_primes($quick ? 2 : 5, 7_500_000) },
-        'Regex/Subst'      => sub {bench_regex_repl($quick ? 8 : 24) },
-        'Regex/Subst utf8' => sub {bench_regex_repl($quick ? 3 : 10, 'utf8') },
-        'Time::Piece'      => sub {bench_timepiece($quick ? 75_000 : 275_000) },
-    },
-    {
-        'Astro'            => ['e71c7ae08f16fe26aea7cfdb72785873', 5.674],
-        'BioPerl Codons'   => ['97c443c099886ca60e99f7ab9df689b5', 8.752],
-        'BioPerl Monomers' => ['d29ed0a5c205c803c112be1338d1f060', 5.241],
-        'CSS::Inliner'     => ['82c1b6de9ca0500a48f8a8df0998df3c', 4.603],
-        'Crypt::JWT'       => ['d41d8cd98f00b204e9800998ecf8427e', 6.451],
-        'DateTime'         => ['b08d2eeb994083b7422f6c9d86fed2c6', 6.198],
-        'Digest'           => ['4b69f6cf0f53cbf6c3444f2f767dd21d', 4.513],
-        'Encode'           => ['PASS 1025',                        5.725],
-        'HTML::FormatText' => ['8c2589f0a5276252805e11301fc2ab56', 4.756],
-        'Imager'           => ['8829cb3703e884054eb025496f336c63', 6.783],
-        'JSON::XS'         => ['PASS',                             5.388],
-        'Math::DCT'        => ['766e3bfd7a2276f452bb3d1bd21939bc', 7.147],
-        'Math::MatrixReal' => ['4606231b1309fb21ae1223fa0043fd76', 4.293],
-        'Moose'            => ['d1cb92c513f6378506dfa11f694cffac', 4.968],
-        'Moose prove'      => ['PASS',                             7.974],
-        'Primes'           => ['4266f70a7a9efb3484cf5d98eba32244', 3.680],
-        'Regex/Subst'      => ['30ce365b25f3d597578b3bdb14aa3f57', 4.652],
-        'Regex/Subst utf8' => ['857eb4e63a4d174ca4a16fe678f7626f', 5.703],
-        'Time::Piece'      => ['5d393b000a42a626bad7208f8be4121e', 5.907],
+    return {               # idx : 0 = result, 1 = ref time, 2 = func, 3 = quick test, 4 = normal test
+        'Astro'            => ['e71c7ae08f16fe26aea7cfdb72785873', 5.674, \&bench_astro, 20000, 80000],
+        'BioPerl Codons'   => ['97c443c099886ca60e99f7ab9df689b5', 8.752, \&bench_bioperl_codons, 3, 5, 1],
+        'BioPerl Monomers' => ['d29ed0a5c205c803c112be1338d1f060', 5.241, \&bench_bioperl_mono, 6, 20],
+        'Crypt::JWT'       => ['d41d8cd98f00b204e9800998ecf8427e', 6.451, \&bench_jwt, 250, 900],
+        'CSS::Inliner'     => ['82c1b6de9ca0500a48f8a8df0998df3c', 4.603, \&bench_css, 2, 5],
+        'DateTime'         => ['b08d2eeb994083b7422f6c9d86fed2c6', 6.198, \&bench_datetime, 5000, 15000],
+        'Digest'           => ['4b69f6cf0f53cbf6c3444f2f767dd21d', 4.513, \&bench_digest, 50, 250],
+        'Encode'           => ['PASS 1025',                        5.725, \&bench_encode, 40, 120],
+        'HTML::FormatText' => ['8c2589f0a5276252805e11301fc2ab56', 4.756, \&bench_formattext, 4, 10],
+        'Imager'           => ['8829cb3703e884054eb025496f336c63', 6.783, \&bench_imager, 4, 16],
+        'JSON::XS'         => ['PASS',                             5.388, \&bench_json, 600, 2200],
+        'Math::DCT'        => ['766e3bfd7a2276f452bb3d1bd21939bc', 7.147, \&bench_dct, 25000, 100_000],
+        'Math::MatrixReal' => ['4606231b1309fb21ae1223fa0043fd76', 4.293, \&bench_matrixreal, 200, 650],
+        'Moose'            => ['d1cb92c513f6378506dfa11f694cffac', 4.968, \&bench_moose, 10_000, 30_000],
+        'Moose prove'      => ['PASS',                             7.974, \&bench_moose_prv, 0.5, 1],
+        'Primes'           => ['4266f70a7a9efb3484cf5d98eba32244', 3.680, \&bench_primes_m, 2, 5],
+        'Regex/Subst'      => ['30ce365b25f3d597578b3bdb14aa3f57', 4.652, \&bench_regex_asc, 8, 24],
+        'Regex/Subst utf8' => ['857eb4e63a4d174ca4a16fe678f7626f', 5.703, \&bench_regex_utf8, 3, 10],
+        'Time::Piece'      => ['5d393b000a42a626bad7208f8be4121e', 5.907, \&bench_timepiece, 75_000, 275_000],
     };
 }
 
@@ -375,7 +353,7 @@ sub suite_run {
 
 sub calc_scalability {
     my ($opt, $stats1, $stats2) = @_;
-    my ($benchmarks, $bench_results) = benchmark_list($opt->{quick});
+    my $benchmarks = benchmark_list();
     my $threads = $stats2->{threads}/$stats1->{threads};
     my $display = $opt->{time} ? 'times' : 'scores';
     $opt->{f} = $opt->{time} ? '%.3f' : '%5.0f';
@@ -406,9 +384,11 @@ sub calc_scalability {
 
 sub run_iteration {
     my ($opt, $stats) = @_;
-    my ($benchmarks, $bench_results) = benchmark_list($opt->{quick});
+    my $benchmarks = benchmark_list();
     my $title = $opt->{time} ? 'Time (sec)' : 'Score';
-    print pad_to("Benchmark").pad_to($title)."Pass/Fail\n";
+    print pad_to("Benchmark").pad_to($title);
+    print "Pass/Fail" unless $opt->{time};
+    print "\n";
     my ($total_score, $total_time, $i) = (0, 0, 0);
     foreach my $bench (sort keys %$benchmarks) {
         next if $opt->{skip_bio} && $bench =~ /Monomers/;
@@ -421,8 +401,8 @@ sub run_iteration {
             require Bio::SeqIO;
             require Bio::Tools::SeqStats;
          }
-        my ($time, $res) = mce_bench_run($opt, $benchmarks->{$bench}, $opt->{quick} ? undef : $bench_results->{$bench});
-        my $score = int(1000*$opt->{threads}*$bench_results->{$bench}->[1]/($time || 1)+0.5);
+        my ($time, $res) = mce_bench_run($opt, $benchmarks->{$bench});
+        my $score = int(1000*$opt->{threads}*$benchmarks->{$bench}->[1]/($time || 1)+0.5);
         $total_score += $score;
         $total_time += $time;
         $i++;
@@ -430,7 +410,9 @@ sub run_iteration {
         push @{$stats->{$bench}->{scores}}, $score;
         my $d = $stats->{$bench}->{$opt->{time} ? 'times' : 'scores'}->[-1];
         $stats->{$bench}->{fail}++ if $res ne 'Pass';
-        print pad_to("$bench:").pad_to(sprintf($opt->{f}, $d))."$res\n";
+        print pad_to("$bench:").pad_to(sprintf($opt->{f}, $d));
+        print "$res" unless $opt->{time};
+        print "\n";
         sleep $opt->{sleep} if $opt->{sleep};
     }
     die "No tests to run\n" unless $i;
@@ -441,14 +423,15 @@ sub run_iteration {
 }
 
 sub mce_bench_run {
-    my $opt = shift;
-    my @params = @_;
-    return bench_run(@params) if $opt->{no_mce};
+    my $opt       = shift;
+    my $benchmark = shift;
+    $benchmark->[3] = $benchmark->[4] unless $opt->{quick};
+    return bench_run($benchmark) if $opt->{no_mce};
 
     my @stats = mce_loop {
         my ($mce, $chunk_ref, $chunk_id) = @_;
         for (@{$chunk_ref}) {
-            my ($time, $res) = bench_run(@params);
+            my ($time, $res) = bench_run($benchmark);
             MCE->gather([$time, $res]);
         }
     }
@@ -464,13 +447,13 @@ sub mce_bench_run {
 }
 
 sub bench_run {
-    my ($bench, $res, $srand) = @_;
+    my ($benchmark, $srand) = @_;
     $srand //= 1;
     srand($srand); # For repeatability
     my $t0   = Time::HiRes::time();
-    my $out  = $bench->();
+    my $out  = $benchmark->[2]->($benchmark->[3]);
     my $time = sprintf("%.3f", Time::HiRes::time()-$t0);
-    my $r    = !$res ? 'N/A' : $out eq $res->[0] ? 'Pass' : "Fail ($out)";
+    my $r    = $out eq $benchmark->[0] ? 'Pass' : "Fail ($out)";
     return $time, $r;
 }
 
@@ -486,8 +469,8 @@ sub bench_astro {
 }
 
 sub bench_bioperl_codons {
-    my $iter = shift;
     my $skip = shift;
+    my $iter = shift;
     my $d    = Digest->new("MD5");
     my $file = catfile($datadir, "gbbct5.seq");
     foreach (1..$iter) {
@@ -634,7 +617,6 @@ sub bench_encode {
 
 sub bench_imager {
     my $iter = shift;
-    my $str  = _read_wiki_files();
     my $d    = Digest->new("MD5");
 
     my $data;
@@ -808,7 +790,7 @@ sub bench_moose {
     return md5_hex("$cnt objects");
 }
 
-sub bench_moose_tests {
+sub bench_moose_prv {
     my $iter = shift;
     my $tdir = catfile($datadir, 't');
     my $result;
@@ -825,6 +807,11 @@ sub bench_moose_tests {
     }
 }
 
+sub bench_primes_m {
+    my $iter = shift;
+    return bench_primes($iter, 7_500_000);
+}
+
 sub bench_primes {
     my $iter = shift;
     my $max  = shift;
@@ -833,12 +820,22 @@ sub bench_primes {
     return md5_hex(scalar(@primes)." primes up to $max");
 }
 
-sub bench_regex_repl {
+sub bench_regex_asc {
+    my $iter = shift;
+    return bench_regex_subst($iter, '');
+}
+
+sub bench_regex_utf8 {
+    my $iter = shift;
+    return bench_regex_subst($iter, 'utf8');
+}
+
+sub bench_regex_subst {
     my $iter  = shift;
-    my $enc   = shift || '';
+    my $enc   = shift;
     my $str   = _read_wiki_files($enc);
     my $match = bench_regex($str, $iter);
-    my $repl  = bench_replace($str, $iter);
+    my $repl  = bench_subst($str, $iter);
     return md5_hex($match, $repl);
 }
 
@@ -855,7 +852,7 @@ sub bench_regex {
     return "$count Matched";
 }
 
-sub bench_replace {
+sub bench_subst {
     my $str  = shift;
     my $iter = shift;
     my $count;
@@ -890,19 +887,19 @@ sub bench_timepiece {
 
 sub total_stats {
     my ($opt, $stats) = @_;
-    my ($benchmarks, $bench_results) = benchmark_list($opt->{quick});
+    my $benchmarks = benchmark_list();
     my $display = $opt->{time} ? 'times' : 'scores';
     my $title   = $opt->{time} ? 'Time (sec)' : 'Score';
     print "Aggregates:\n".pad_to("Benchmark",24).pad_to("Avg $title").pad_to("Min $title").pad_to("Max $title");
-    print pad_to("Pass %") unless $opt->{quick};
+    print pad_to("Pass %") unless $opt->{time};
     print "\n";
     foreach my $bench (sort keys %$benchmarks) {
         next unless $stats->{$bench}->{$display};
         my $str = calc_stats($opt, $stats->{$bench}->{$display});
         print pad_to("$bench:",24).$str;
         print pad_to(
-            sprintf("%d%%", 100 * ($opt->{iter}-($stats->{$bench}->{fail} || 0)) / $opt->{iter}))
-            unless $opt->{quick};
+            sprintf("%d", 100 * ($opt->{iter}-($stats->{$bench}->{fail} || 0)) / $opt->{iter}))
+            unless $opt->{time};
         print "\n";
     }
     my $str = calc_stats($opt, $stats->{total}->{$display});
@@ -1033,7 +1030,7 @@ sub rand_hash {
 
 # modified from https://github.com/famzah/langs-performance/blob/master/primes.pl
 sub _get_primes {
-    my $n = shift || 100_000_000;
+    my $n = shift;
     my @s = ();
     for (my $i = 3; $i < $n + 1; $i += 2) {
         push(@s, $i);
